#!/arch/unix/bin/perl5

################################################################
# $Id: yar2r.pl,v 1.13 1999/12/20 22:21:38 danielr Exp $
################################################################
# Daniel R. <danielr@ccs.neu.edu> [http://www.ccs.neu.edu/home/danielr/]
#
# Please see the README for others who have contributed
#
# Started: Nov 19, 1999
# Current: $Date: 1999/12/20 22:21:38 $
#
# YaR2R (Yet another REQ to RT converter)
#
# Hopefully this one will do the right thing
################################################################
# Needed for getting at routines
package rt;

################################################################
# RT imports - Modify these for your local install
################################################################
# Change this to where you have RT's lib directory
push( @INC, "/share/adm/packages/rt-1.0.1/lib" );

# Change this to the location of RT's config.pm
require "/share/adm/packages/rt-1.0.1/etc/config.pm";
require rt::database::manipulate;

# Needed for RT time setting when strict is on
use vars '$time';

# Program Safely
use strict;

# Parse Date
use Date::Manip;

################################################################
# GLOBALS - Modify these for your local install
################################################################

# Domain to append to usernames that don't include an @
my( $DOMAIN ) = "example.edu";

# What queue to import into
my( $QUEUE ) = "systems";

# Default RT user for bad username (see below)
my( $RTUSER ) = "root";

# Separator used in REQ files
my( $SEPARATOR ) = "=" x 75;

# Should errors be generated for unknown users?
# See the README for more details
my( $USER_WARN ) = 0;

# Mapping of usernames to list of users in the RT database that can
# modify the queue. Also include valid to valid mappings. Example:
# my( %USERS ) = ( "Dave", "dave",
#  		   "Jay", "jay",
#		   "JeffC", "jeffc",
#		   "danielr", "danielr",
#		   "dave", "dave",
#		   "jay", "jay",
#		   "jeffc", "jeffc" );
my( %USERS ) = ();

# Set this priorities that might be used. Default will be used if a
# found priority doesn't match one of the other values.
my( %PRIORITIES ) = ( "low", 25,
		      "normal", 50,
		      "high", 75,
		      "default", 50 );

# Actually import the REQs into RT
my( $COMMIT ) = 0;

# Report what REQ is being worked on
my( $VERBOSE ) = 1;

# Report on each transaction in a REQ file as it is being parsed
my( $DEBUG ) = 0;
$| = 1 if ( $DEBUG || $VERBOSE );

################################################################
# What is being imported - Modify these for your local install
################################################################

# Uncomment these lines if you are fixing a problem REQ file and want
# to see just where the parsing is failing
#$DEBUG = 1;
#&parseREQ( "/net/request/active/19708" );
#exit;

# Where the queues live
my( @directory ) = ( "/net/request/resolved",
		     "/net/request/active" );

my( $directory, $req );
foreach $directory ( @directory ) {
    opendir( REQ, $directory );
    my( @files ) = sort( { $a <=> $b } grep( /^\d+$/, readdir( REQ ) ) );
    close( REQ );
    
    foreach $req ( @files ) {
	&parseREQ( "$directory/$req" );
    }
}

################################################################
# NOTHING SHOULD NEED TO BE EDITED BELOW THIS POINT
################################################################

################################################################
# REQ File Parsing
################################################################
# This is a recursive function in order to handle merged REQs
# correctly. It either expects a filename or an array reference
sub parseREQ {
    my( $req, $inMerge ) = @_;
    my( $line );
    
    ################
    # Read the REQ
    ################
    # It is stored in an array for easy line look ahead and easy data
    # passing during recursion
    if ( ! $inMerge ) {
	print "Opening REQ file \"$req\"\n" if ( $DEBUG || $VERBOSE );
	open( REQ, $req ) || &error( "Unable to read \"$req\" : $!" );
	my( @req ) = <REQ>;
	close( REQ );
	$req = \@req;
    }

    ################
    # REQ Header
    ################
    # get the first real line of the file
    while( ( $line = shift( @ { $req } ) ) && $line =~ /^\s*$/ ) {}
    chomp( $line );
    
    #
    # These are the fields at the top of a new REQ
    # Fields marked with an '*' are ignored because future REQ commands
    # will update them
    # Did REQ ever really use keywords or areas?
    #
    #   X-Request-Number: 19461
    # * X-Request-Owner: 
    #   X-Request-User: user@example.com
    #   X-Request-Date: Sat Nov 20 12:20:02 1999 (943118402)
    # * X-Request-Due: 
    # * X-Request-Status: open
    # * X-Request-Priority: normal
    # * X-Request-Updated: Sat Nov 20 12:20:02 1999 (943118402)
    # * X-Request-Notified: 
    # * X-Request-Keywords: 
    # * X-Request-Areas: 
    #
    # First line should be an X-Request line. It's possible that it
    # doesn't have proper case so ignore it.
    &error( "First line of file is not an X-Request field. Is this a valid REQ?" ) if ( $line !~ /^X-Request-/i );
    
    my( $number, $date );
    my( $action ) = "new";
    my( @actions ) = ( $action );
    my( %actions );

    # Parse all of the X-Request fields
    while( $line =~ /^X-Request-/i ) {
	my( $command, $value ) = &parseXRequest( $line, $req );
	
	if ( $command eq "number" ) {
	    # In the case of a merged REQ multiple numbers are listed
	    # Only use the first one
	    $value =~ s/,.*//;
	    $number = $value;
	    print "Now working on REQ# $number\n" if ( $DEBUG || $VERBOSE );
	} elsif ( $command eq "merged" ) {
	    &error( "Merged header shouldn't be found if in a merge." )
	      if ( $inMerge );

	    # This REQ was merged with another one so ignore it
	    print "REQ merged into another one, skipping.\n"
	      if ( $DEBUG || $VERBOSE );
	    return;
	} elsif ( $command eq "user" ) {
	    $actions{ $action }{ "user" } = &parseUser( $value );
	    print "\tSubmitted by: ", $actions{ $action }{ "user" }, "\n" 
	      if ( $DEBUG );
	} elsif ( $command eq "date" ) {
	    $date = &parseDate( $value );
	    print "\tSubmitted on: ", scalar( localtime( $date ) ), "\n"
	      if ( $DEBUG );
	}
	# ignore all of the other headers
	
	$line = shift( @ { $req } );
	chomp( $line );
    }
    
    # A sanity check that a valid REQ header was found
    &error( "REQ number or date not found." ) 
      if ( $number eq "" || $date eq "" );

    ################
    # Message Body
    ################
    my( $done ) = 0;
    while( ! $done ) {
	my( @body ) = ();
	push( @body, $line );
	
	# Make sure something that looks like the req separator is
	# really followed by an X-Request header, just in case the
	# REQ's body contains a series of ='s
	print "Reading body " if ( $DEBUG );
	while( ( $line = shift( @ { $req } ) ) &&
	       ( $line !~ /^$SEPARATOR/ ||
		 $ { $req }[ 0 ] !~ /^X-Request/i ) ) {
	    print "." if ( $DEBUG );
	    chomp( $line );
	    push( @body, $line );
	}
	print "\n" if ( $DEBUG );

	# Remove possible merge text from body or blank line(s) at the
	# top and bottom
	while( scalar( @body ) && ( $body[ $#body ] =~ /^\s*$/ || 
				    $body[ $#body ] =~ 
				    /End\sof\smerge\sof\s\d/ ) ) {
	    pop( @body );
	}
	while( scalar( @body ) && $body[ 0 ] =~ /^\s*$/ ) {
	    shift( @body );
	}
	
	# There are additional commands to deal with
	if ( $line =~ /^$SEPARATOR/ ) {
	    # Act on data already found
	    &commit( $number, $date, \@body, \@actions, \%actions );
	    @actions = ();
	    %actions = ();

	    $line = shift( @ { $req } );
	    chomp( $line );

	    # Figure out what the next action is
	    while( $line =~ /^X-Request-/i ) {
		my( $command, $value ) = &parseXRequest( $line, $req );
		
		if ( $command eq "action" ) {
		    my( $type, @values ) = &parseAction( $value );
		    if ( $type eq "merge" ) {
			if ( $values[ 1 ] == $number ) {
			    # At the end of the merge.  Nothing to
			    # save so get out quickly
			    return;
			} else {
			    # Start of new merge

			    # 1. Save who did the merge
			    $action = $type;
			    push( @actions, $action );
			    $actions{ $action }{ "user" } = shift( @values );
			    $actions{ $action }{ "values" } = \@values;

			    # 2. Recursively process the merged REQ
			    &parseREQ( $req, 1 );

			    # 3. The next line is an X-Request line,
			    # because that is where the recursion
			    # stopped, let the outer loop parse it
			}
		    } else {
			# Normal action, save the information
			$action = $type;
			push( @actions, $action );
			$actions{ $action }{ "user" } = shift( @values );
			$actions{ $action }{ "values" } = \@values;
		    }
		} elsif ( $command eq "acted" ) {
		    # Grab when the action was done
		    $date = &parseDate( $value );
		} else {
		    &error( "Bad X-Request after separator. REQ# $number." );
		}
		$line = shift( @ { $req } );
		chomp( $line );
	    }
	} else {
	    # Reached the end of the file commit any remaining actions
	    &commit( $number, $date, \@body, \@actions, \%actions );

	    $done = 1;
	}
    }
}

################################################################
# Parse X-Request Line
################################################################
# What this handles:
#
# X-Request-?????: ?????
#
# What this returns:
# 
# This divides up an X-Request line into a command and value
# It also automatically tries to fix the common subject line error
#
sub parseXRequest {
    my( $line, $req ) = @_;

    chomp( $line );
    
    # Try to repair the 'Subject changed to' newline problem by
    # appending the next line to this one
    if ( ( $line =~ /^X-Request-Action:\sSubject\schanged\sto\s.*/i )
	 && ( $ { $req }[ 0 ] =~ /^".*/ ) )  {
	print "Joining \"$line\" to next.\n" if ( $DEBUG || $VERBOSE );
	$line .= shift( @ { $req } );
	chomp( $line );
	print "It now looks like \"$line\"\n" if ( $DEBUG || $VERBOSE );
    }

    my( $command, $value ) = split( /:/, $line, 2 );
    
    # Some REQs have munged upper/lower case headers so force lowercase
    $command = lc( substr( $command, 10 ) );

    # Clean up the value some
    $value =~ s/^\s*//;
    $value =~ s/\s*$//;
    
    return( $command, $value );
}

################################################################
# Parse X-Request-User Value
################################################################
# What this handles:
#
# user@example.com,user2@example.com
# User Name <user>
# user@example.com
# user
# user>" <user@example.com
# user (User Name)
#
# What this returns:
#
# A value in the form user@example.com
#
sub parseUser {
    my( $value ) = shift;

    # Kill multiple users due to merging
    $value =~ s/,.*//;
    
    # Handle > < error
    $value =~ s/\>.*// if ( $value =~ /^[^<]+\>/ );
    
    # Extract eMail out of <>
    $value =~ s/.*\<([^>]*)>.*/$1/;
    
    # Kill () based eMails
    $value =~ s/\(.*//;
    
    # Make sure we don't have any fluff
    $value =~ s/^\s*//;
    $value =~ s/\s*$//;
    
    # Append default domain for unqualified names
    $value .= "\@$DOMAIN" if ( $value !~ /\@/ );
    
    # Lastly lc domain to improve username consistency
    $value =~ s/(\@.*)/\L$1\E/;
    
    return( $value );
}

sub testParseUser {
    my( @tests ) = ( 'user@example.com,user2@example.com',
		     'User Name <user>',
		     'user@example.com',
		     'user',
		     'user>" <user2@example.com',
		     'user (User Name)' );
    
    my( $test );
    foreach $test ( @tests ) {
	print "Testing : $test\n";
	print "\t", &parseUser( $test ), "\n";
    }
}    

################################################################
# Parse X-Request-Date Value
################################################################
# What this handles:
#
# Tue, 1 Feb 1994 18:39:09 -0500 (760109949)
# Mon, 11 Apr 94 08:21:19 -0400 (766038079)
# Sat, 19 Feb 1994 19:05:45 -0500 (EST)
# Wed, 13 Apr 1994 16:04:15 -0400 (EDT)
# Wed, 6 Apr 1994 17:34:11 -0400
# Tue May 31 17:01:17 1994 (770418077)
# Wed May 11 12:24:09 1994 EDT (768673449)
# Wed, 20 Apr 94 07:10:53 BST (766822253)
# Tue Jul 16 20:17:43 EDT 1996 (837562663)
#
# What it returns:
#
# Epoch time based on local time zone
#
# What is assumes:
#
# That the time string given is for the local time zone
#
# Known bugs:
#
# Doesn't handle DST very well
#
sub parseDate {
    my( $value ) = shift;

    # Remove anything in ()
    $value =~ s/\(.*//;
    
    # Remove anything after a - or +
    $value =~ s/[-+].*//;

    # Remove anything that looks like a TZ
    $value =~ s/[A-Z][A-Z][A-Z]//;

    # Pass it over to Date::Manip
    my( $date ) = &ParseDate( $value );
    &error( "Unable to parse date from \"$value\"!") if ( ! $date );

    # Return it in local time zone epoch time
    return( &UnixDate( $date, "%s" ) );
}

sub testParseDate {
    my( @tests ) = ( 'Tue, 1 Feb 1994 18:39:09 -0500 (760109949)',
		     'Mon, 11 Apr 94 08:21:19 -0400 (766038079)',
		     'Sat, 19 Feb 1994 19:05:45 -0500 (EST)',
		     'Wed, 13 Apr 1994 16:04:15 -0400 (EDT)',
		     'Wed, 6 Apr 1994 17:34:11 -0400',
		     'Tue May 31 17:01:17 1994 (770418077)',
		     'Wed May 11 12:24:09 1994 EDT (768673449)',
		     'Wed, 20 Apr 94 07:10:53 BST (766822253)',
		     'Tue Jul 16 20:17:43 EDT 1996 (837562663)' );

    my( $test );
    foreach $test ( @tests ) {
	print "Testing : $test\n";
	print "\t", &parseDate( $test ), "\n";
	print "\t", scalar( localtime( &parseDate( $test ) ) ), "\n";
    }
}

################################################################
# Parse Subject Line
################################################################
# What this handles:
#
# Subject: [Req #19461] Input Jam on Echer
# [Req #19461] Input Jam on Echer
# Subject: [Req #10143] Re:  Macaulay 2
# [Req #10143] Re:  Macaulay 2
# Re: [Req #9075] URGENT: zero-length files appearing in /proj/qwer
#
# What this returns:
#
# A subject line with "Subject:" and any REQ# info removed
#
sub parseSubject {
    my( $value ) = shift;
    
    # Remove Subject
    $value =~ s/^\s*subject:\s*//i;
    
    # Remove [REQ: #???]
    $value =~ s/\[req[^\]]*\]//i;

    # Remove RE: info
    $value =~ s/^\s*re:\s*//i;

    # Clean it up some
    $value =~ s/^\s*//;
    $value =~ s/\s*$//;

    # Remove multiple spaces (not for everyone)
    $value =~ s/(\s)\s+/$1/g;

    return( $value );
}

sub testParseSubject {
    my( @tests ) = ( 'Subject: [Req #19461] Input Jam on Echer ',
		     '[Req #19461] Input Jam on Echer',
		     'Subject: [Req #10143] Re:  Macaulay 2',
		     '[Req #10143] Re:  Macaulay 2 ',
		     'Re: [Req #9075] URGENT: zero-length files appearing in /proj/qwer' );
    
    my( $test );
    foreach $test ( @tests ) {
	print "Testing : $test\n";
	print "\t\"", &parseSubject( $test ), "\"\n";
    }
}

################################################################
# Parse X-Request-Action Value
################################################################
# What this handles:
#
# # merged into # by USER.
# Comments added by USER.
# Given to USER by USER.
# Opened by USER.
# Ownership changed to USER by USER. (bad wording, actually changes user)
# Priority set to PRI by USER.
# Resolved by USER.
# Stalled by USER.
# Subject changed to "SUBJECT" by USER.
# Taken by USER.
# User changed to USER by USER.
# USER notified by USER.
# Un-taken by USER.
#
# What this returns:
#
# type of action
# USER who issued the action
# 0 or more values (varies by action)
#
sub parseAction {
    my( $value ) = shift;
    my( $type, @values, $temp );

    $value =~ s/\s*$//;

    # Figure out who did the action
    ( $temp ) = ( $value =~ /\sby\s(.*)\.$/ );
    &error( "Unable to find user in \"$value\"." ) if ( $temp eq "" );
    push( @values, &parseUser( $temp ) );
    
    # # merged into # by USER.
    if ( $value =~ /\smerged\sinto\s/i ) {
	$type = "merge";

	( $temp ) = ( $value =~ /^(\d+)/ );
	&error( "Unable to find number in \"$value\"." ) if ( $temp eq "" );
	push( @values, $temp );

	( $temp ) = ( $value =~ /(\d+)\sby/ );
	&error( "Unable to find number in \"$value\"." ) if ( $temp eq "" );
	push( @values, $temp );

	# Comments added by USER.
    } elsif ( $value =~ /^comments\sadded/i ) {
	$type = "comment";

	# Given to USER by USER.
    } elsif ( $value =~ /^given\sto/i ) {
	$type = "give";

	( $temp ) = ( $value =~ /\sto\s(.*)\sby\s/ );
	&error( "Unable to find user in \"$value\"." ) if ( $temp eq "" );
	push( @values, &parseUser( $temp ) );

	# Opened by USER.
    } elsif ( $value =~ /^opened\sby/i ) {
	$type = "open";

	# Ownership changed to USER by USER. (bad wording, actually
	# changes user)
    } elsif ( $value =~ /^ownership\schanged/i ) {
	$type = "user";

	( $temp ) = ( $value =~ /\sto\s(.*)\sby\s/ );
	&error( "Unable to find user in \"$value\"." ) if ( $temp eq "" );
	push( @values, &parseUser( $temp ) );

	# Priority set to PRI by USER.
    } elsif ( $value =~ /^priority\sset/i ) {
	$type = "priority";
	
	( $temp ) = ( $value =~ /\sto\s(.*)\sby\s/ );
	&error( "Unable to find priority in \"$value\"." ) if ( $temp eq "" );
	$temp = "default" if ( ! exists( $PRIORITIES{ $temp } ) );
	push( @values, $PRIORITIES{ $temp } );
	
	# Resolved by USER.
    } elsif ( $value =~ /^resolved\sby/i ) {
	$type = "resolve";

	# Stalled by USER.
    } elsif ( $value =~ /^stalled\sby/i ) {
	$type = "stall";

	# Subject changed to "SUBJECT" by USER.
    } elsif ( $value =~ /^subject\schanged/i ) {
	$type = "subject";

	( $temp ) = ( $value =~ /\sto\s\"(.*)\"\sby\s/ );
	&error( "Unable to find subject in \"$value\"." ) if ( $temp eq "" );
	push( @values, &parseSubject( $temp ) );

	# Taken by USER.
    } elsif ( $value =~ /^taken\sby/i ) {
	$type = "take";

	# User changed to USER by USER.
    } elsif ( $value =~ /^user\schanged/i ) {
	$type = "user";

	( $temp ) = ( $value =~ /\sto\s(.*)\sby\s/ );
	&error( "Unable to find user in \"$value\"." ) if ( $temp eq "" );
	push( @values, &parseUser( $temp ) );

	# USER notified by USER.
    } elsif ( $value =~ /\snotified\sby\s/i ) {
	$type = "notify";

	( $temp ) = ( $value =~ /^(.*)\snotified\s/ );
	&error( "Unable to find user in \"$value\"." ) if ( $temp eq "" );
	push( @values, &parseUser( $temp ) );
	
	# Un-taken by USER.
    } elsif ( $value =~ /^un-taken\sby/i ) {
	$type = "untake";

    } else {
	&error( "No known action \"$value\"." );
    }

    return( $type, @values );
}

sub testParseAction {
    my( @tests ) = ( '4 merged into 30 by abcd.',
		     'Comments added by asdf@example.com.',
		     'Comments added by poiu.',
		     'Given to qwer by fghj.',
		     'Given to rty by qwer@example.com .',
		     'Opened by sdfg.',
		     'Priority set to low by tyui.',
		     'Resolved by nbvc.',
		     'Stalled by dfgh.',
		     'Subject changed to "Emacs: emacs info missing" by xcvb.',
		     'Subject changed to "[Req #3]  Re: print and disk" by ghjk.',
		     'Taken by xcvb.',
		     'Un-taken by gfds.',
		     'Qwer Tyui <zxcv> notified by poiu.',
		     'asdf@example.com notified by fghj@example.com.' );

    my( $test );
    foreach $test ( @tests ) {
	print "Testing : $test\n";
	print "\t\"", join( "\"\n\t\"", &parseAction( $test ) ), "\"\n";
    }
}

################################################################
# User Routines
################################################################
sub extractUser {
    my( $user ) = shift;

    if ( $user =~ /\Q$DOMAIN\E/ ) {
	$user =~ s/@[^@]*$//;
    }

    return( $user );
}

sub isUser {
    my( $user ) = &extractUser( shift( @_ ) );
    return( exists( $USERS{ $user } ) ? 1 : 0 );
}

sub getUser {
    my( $user ) = &extractUser( shift( @_ ) );
    &error( "\"$user\" is not in local domain" ) if ( $user =~ /\@/ && 
						      $USER_WARN );
    
    if ( ! exists( $USERS{ $user } ) ) {
	&error( "\"$user\" doesn't exist\n" ) if ( $USER_WARN );
	$USERS{ $user } = $RTUSER;
    }
    return( $USERS{ $user } );
}
	
################################################################
# Commit something into RT
################################################################
# 
# Possible actions are:
# new
# merge
# comment
# give
# open
# owner
# priority
# resolve
# stall
# subject
# take
# owner
# notify
# untake

# $action, $number, $user, $date, join( "\n", @body ) );
sub commit {
    my( $number, $date, $body, $actions, $actionData ) = @_;

    &error( "No actions passed to commit for REQ# $number." )
      if ( ! scalar( @ { $actions } ) );
    
    $time = $date;
    
    my( $action );

    foreach $action ( @ { $actions } ) {
	my( $user ) = $ { $actionData }{ $action }{ "user" };
	my( @values ) = 
	  ( ref( $ { $actionData }{ $action }{ "values" } ) eq "ARRAY" ) ? 
	    @ { $ { $actionData }{ $action }{ "values" } } : ();

	my( $line, $subject );
	foreach $line ( @ { $body } ) {
	    if ( $line =~ /^subject:/i ) {
		$subject = &parseSubject( $line );
		last;
	    }
	}
	
 	if ( $action eq "new" ) {
	    &importRequest( $number, $user, $subject, $date, 
			    join( "\n", @ { $body } ) );
	} elsif ( $action eq "merge" ) {
	    &mergeRequest( &getUser( $user ), $values[ 0 ], $values[ 1 ] );
	} elsif ( $action eq "comment" ) {
	    &importCorrespondance( $number, join( "\n", @ { $body } ), 
				   $subject, $user );

	    # If the user is valid have them notify
	    &notify( $number, $date, &getUser( $user ) ) 
	      if ( $COMMIT && &isUser( $user ) );

	} elsif ( $action eq "give" ) {
	    if ( &getUser( $values[ 0 ] ) eq &getUser( $user ) ) {
		&stealRequest( $number, &getUser( $user ), &getUser( $user ) );
	    } else {
		&giveRequest( $number, &getUser( $values[ 0 ] ), 
			      &getUser( $user ) );
	    }
	} elsif ( $action eq "open" ) {
	    if ( &isUser( $user ) ) {
		&openRequest( $number, &getUser( $user ) );
	    } else {
		&openRequest( $number, $RTUSER );
	    }		
	} elsif ( $action eq "priority" ) {
	    &changePriority( $number, $values[ 0 ], &getUser( $user ) );
	} elsif ( $action eq "resolve" ) {
	    &resolveRequest( $number, &getUser( $user ) );
	} elsif ( $action eq "stall" ) {
	    &stallRequest( $number, &getUser( $user ) );
	} elsif ( $action eq "subject" ) {
	    if ( &isUser( $user ) ) {
		&changeSubject( $number, $values[ 0 ], &getUser( $user ) );
	    } else {
		&changeSubject( $number, $values[ 0 ], $RTUSER );
	    }		
	} elsif ( $action eq "notify" ) {
	    # Not 100% correct but it works out better
	    &importCorrespondance( $number, join( "\n", @ { $body } ), 
				   $subject, $user );
	    
	    # update notify time
	    &notify( $number, $date, &getUser( $user ) ) if ( $COMMIT );
	} elsif ( $action eq "take" ) {
	    &takeRequest( $number, &getUser( $user ) );
	} elsif ( $action eq "untake" ) {
	    &untakeRequest( $number, &getUser( $user ) );
	} elsif ( $action eq "user" ) {
	    &changeRequester( $number, $values[ 0 ], &getUser( $user ) );
	}
	$time++;
    }
}

################################################################
# Import Request
################################################################
# These are the fields and the order that RT needs
#
#  1. Queue ID
#  2. Serial Number
#  3. Area
#  4. Requesters
#  5. Alias
#  6. Owner
#  7. Subject
#  8. Final Priority
#  9. Priority
# 10. Status
# 11. Date Created
# 12. Date Told
# 13. Date Due
# 14. Content
# 15. Current User
#
sub importRequest {
    my( $number, $requester, $subject, $date, $body ) = @_;
    
    if ( $COMMIT ) {
	my( @return ) = &import_request( $QUEUE, $number, "", $requester, 
					 "", "", $subject, 0, 
					 $PRIORITIES{ "default" }, "open",
					 $date, 0, 0, $body, $RTUSER );
	&error( "Import on $number failed for $RTUSER : "
		. join( " ", @return ) ) if ( $return[ 0 ] == 0 );
    } 
    print "Importing $number\n" if ( $DEBUG );
}

################################################################
# Merge Requests
################################################################
# These are the fields and the order that RT needs
# 
# 1. Serial Number
# 2. Merge into
# 3. Current User
#
sub mergeRequest {
    my( $user, $source, $destination ) = @_;
    
    if ( $COMMIT ) {
	my( @return ) = &merge( $source, $destination, $user );
	&error( "Merge $source into $destination failed for $user : " . 
		join( " ", @return ) ) if ( $return[ 0 ] == 0 );
    } 
    print "Merged $source into $destination by $user\n" if ( $DEBUG );
}

################################################################
# Import Correspondence
################################################################
# These are the fields and the order that RT needs
# 
# 1. Serial Number
# 2. Content
# 3. Subject
# 4. Current User
#
sub importCorrespondance {
    my( $number, $body, $subject, $person ) = @_;

    if ( $COMMIT ) {
	my( @return ) = &import_correspondence( $number, $body, 
						$subject, $person );
	&error( "Import correspondence on $number failed for $person : " . 
		join( " ", @return ) ) if ( $return[ 0 ] == 0 );
    } 
    print "Correspondence added to $number by $person\n" if ( $DEBUG );
}

################################################################
# Give Request
################################################################
# These are the fields and the order that RT needs
# 
# 1. Serial Number
# 2. Owner
# 3. Current User
#
sub giveRequest {
    my( $number, $newOwner, $user ) = @_;

    if ( $COMMIT ) {
	my( @return ) = &give( $number, $newOwner, $user );

	# No one owned request so just have the newOwner take it
	if ( $return[ 1 ] =~ /owned\sby/ ) {
	    &takeRequest( $number, $newOwner );
	    return();
	    
	    # The actor doesn't own the request so have the newOwner
	    # steal it
	} elsif ( $return[ 1 ] =~ /stealing/ ) {
	    &stealRequest( $number, $newOwner );
	    return();
	} else {
	    &error( "Giving $number to $newOwner failed for $user : " . 
		    join( " ", @return ) ) if ( $return[ 0 ] == 0 );
	}
    } 
    print "$number given to $newOwner by $user\n" if ( $DEBUG );
}

################################################################
# Open Request
################################################################
# These are the fields and the order that RT need
# 
# 1. Serial Number
# 2. Current User
#
sub openRequest {
    my( $number, $user ) = @_;
    
    if ( $COMMIT ) {
	my( @return ) = &open( $number, $user );

	# Not checking for error because RT can return 0 even if
	# successful
    } 
    print "$number opened by $user\n" if ( $DEBUG );
}

################################################################
# Change Requester
################################################################
# These are the fields and the order that RT need
# 
# 1. Serial Number
# 2. Requester
# 3. Current User
#
sub changeRequester {
    my( $number, $newRequester, $user ) = @_;
    
    if ( $COMMIT ) {
	my( @return ) = &change_requestors( $number, $newRequester, $user );

	# Not checking for error because RT can return 0 even if
	# successful
    } 
    print "$number requester changed to $newRequester by $user\n" if ( $DEBUG );
}

################################################################
# Change Priority
################################################################
# These are the fields and the order that RT need
# 
# 1. Serial Number
# 2. Priority
# 3. Current User
#
sub changePriority {
    my( $number, $newPriority, $user ) = @_;
    
    if ( $COMMIT ) {
	my( @return ) = &change_priority( $number, $newPriority, $user );

	# Not checking for error because RT can return 0 even if
	# successful
    } 
    print "$number priority changed to $newPriority by $user\n" if ( $DEBUG );
}

################################################################
# Resolve
################################################################
# These are the fields and the order that RT need
# 
# 1. Serial Number
# 2. Current User
#
sub resolveRequest {
    my( $number, $user ) = @_;
    
    if ( $COMMIT ) {
	my( @return ) = &resolve( $number, $user );

	# Not checking for error because RT can return 0 even if
	# successful
    } 
    print "$number resolved by $user\n" if ( $DEBUG );
}

################################################################
# Stall
################################################################
# These are the fields and the order that RT need
# 
# 1. Serial Number
# 2. Current User
#
sub stallRequest {
    my( $number, $user ) = @_;
    
    if ( $COMMIT ) {
	my( @return ) = &stall( $number, $user );

	# Not checking for error because RT can return 0 even if
	# successful
    } 
    print "$number stalled by $user\n" if ( $DEBUG );
}

################################################################
# Change Subject
################################################################
# These are the fields and the order that RT need
# 
# 1. Serial Number
# 2. Subject
# 3. Current User
#
sub changeSubject {
    my( $number, $newSubject, $user ) = @_;
    
    if ( $COMMIT ) {
	my( @return ) = &change_subject( $number, $newSubject, $user );

	# Not checking for error because RT can return 0 even if
	# successful
    } 
    print "$number subject changed to $newSubject by $user\n" if ( $DEBUG );
}

################################################################
# Steal
################################################################
# These are the fields and the order that RT need
# 
# 1. Serial Number
# 2. Current User
#
sub stealRequest {
    my( $number, $user ) = @_;
    
    if ( $COMMIT ) {
	my( @return ) = &steal( $number, $user );

	# No one owned the request so just take it, instead of
	# stealing
	if ( $return[ 1 ] =~ /owned\sby/ ) {
	    &takeRequest( $number, $user );
	    return();
	} else {
	    &error( "Stealing $number failed for $user : " . 
		    join( " ", @return ) ) if ( $return[ 0 ] == 0 );
	}
    } 
    print "$number stolen by $user\n" if ( $DEBUG );
}

################################################################
# Take
################################################################
# These are the fields and the order that RT need
# 
# 1. Serial Number
# 2. Current User
#
sub takeRequest {
    my( $number, $user ) = @_;
    
    if ( $COMMIT ) {
	my( @return ) = &take( $number, $user );

	# Someone else already owns it, so steal it instead
	if ( $return[ 1 ] =~ /stealing/ ) {
	    &stealRequest( $number, $user );
	    return();

	    # Actor already owns it, ignore the action
	} elsif ( $return[ 1 ] =~ /already\sowns/ ) {
	    print "$number taken by $user\n" if ( $DEBUG );
	    return();
	} else {
	    &error( "Taking $number failed for $user : " . 
		    join( " ", @return ) ) if ( $return[ 0 ] == 0 );
	}

    }
    print "$number taken by $user\n" if ( $DEBUG );
}

################################################################
# Untake
################################################################
# These are the fields and the order that RT need
# 
# 1. Serial Number
# 2. Current User
#
sub untakeRequest {
    my( $number, $user ) = @_;
    
    if ( $COMMIT ) {
	my( @return ) = &untake( $number, $user );

	# Actor doesn't own it, so they can't untake it
	if ( $return[ 1 ] =~ /can\suntake/ ) {
	    print "$number untaken by $user\n" if ( $DEBUG );
	    return();

	    # Actor doesn't own it, so they can't untake it
	} elsif ( $return[ 1 ] =~ /stealing/ ) {
	    print "$number untaken by $user\n" if ( $DEBUG );
	    return();

	    # Nobody owns request, so you can't untake it 
	} elsif ( $return[ 1 ] =~ /already\sowns/ ) {
	    print "$number untaken by $user\n" if ( $DEBUG );
	    return();

	} else {	
	    &error( "Untaking $number failed for $user : " . 
		    join( " ", @return ) ) if ( $return[ 0 ] == 0 );
	}
    }
    print "$number untaken by $user\n" if ( $DEBUG );
}

################################################################
# Error Messages
################################################################
sub error {
    my( $error ) = shift;

    chomp( $error );
    
    print "\n***** ERROR: $error\n\n";
    exit() if ( $COMMIT );
}
